# $Id: ui.py,v 1.14 2002/02/17 22:00:19 marijn Exp $

# An air traffic simulation game. 
# Copyright (C) 2000,1,2  Marijn Vriens <marijn@sanity.dhs.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import setup

import os
import libglade
import gtk
import GDK
import gnome.ui

import abswindow
import selectwin
import controlwin
import dialogwin
import radardisplay

class Ui(libglade.GladeXML):
    def genConnects(self, fifoRead, simConnection, metaConnection):
        self._registerUiElements()
        self._showVersion()
        self._threads = self._buildThreadInfoDict(simConnection,
                                                  metaConnection)
        self._fifoDataLength = None
        self._fifoDataLength = self._getFifoDataLength()
        self._fifo = fifoRead
        self._waitForThreadsActiveStatus(self._fifo, self._threads)
        self._setupCallbacks()

    def _registerUiElements(self):
        abswindow.WindowList(self)
        MainWin(self.get_widget("MainWin"))
        PrefWin(self.get_widget("PrefWin"))
        DetailWin(self.get_widget("DetailsWin"))
        FlightWin(self.get_widget("FlightsWin"))
        AboutWin(self.get_widget("AboutWin"))
        controlwin.ControlWin(self.get_widget("ControlWin"))
        selectwin.SelectWin(self.get_widget("SelectWin"))
        dialogwin.DialogWin(self.get_widget("DialogWin"))
        radardisplay.RadarDisplay(self.get_widget("RadarDisplay"))
    def _getFifoDataLength(self):
        if(self._fifoDataLength == None):
            self._fifoDataLength = len(repr(id(self))) + 1 
        return self._fifoDataLength
        
    def _waitForThreadsActiveStatus(self, fifo, producers):
        for i in range(len(producers)):
            data = self._readFifoData(self._fifo)
            #print "Thread %s: %s came online" % (data, producers[data])

    def _waitForThreadsExitStatus(self, fifo, producers):
        #for i in range(len(producers)):
        i = 0
        while(i < len(producers)):
            data = self._readFifoData(fifo)
            #print "Thread %s: %s exited" % (data, producers[data])
            i = i + 1

    def _setupCallbacks(self):
        self._fifoInput = gtk.input_add(self._fifo, GDK.INPUT_READ, self._fifoCallBack)
        #self._idleCallBack = gtk.timeout_add(500, self._idleCallBack)

    def _idleCallBack(self):
        print "Ui::_idleCallBack called"
        for i in self._threads.values():
            i.doCallBacks()
        return 1

    def _fifoCallBack(self, *args):
        #print "Ui::_fifoCallBack: calling doCallBacks on data"
        data = self._readFifoData(self._fifo)
        self._threads[data].doCallBacks()
        #print "Ui::_fifoCallBack: Done"

    def _readFifoData(self, fifo):
        data = os.read(fifo, self._fifoDataLength)
        #print "Ui::_readFifoData: read \"%s\" from fifo" % data
        return data.strip()

    def _buildThreadInfoDict(self, sim, meta):
        threads = {}
        threads[repr(id(sim))]  = sim
        threads[repr(id(meta))] = meta
        return threads

    def destroy(self, widget):
        abswindow.WindowList().hideAllWindows()
        if(gtk.events_pending()):
            gtk.mainiteration()

        gtk.input_remove(self._fifoInput)
        for i in self._threads.values():
            #print "requesting exit from %s" % i
            i.requestExit()
        self._waitForThreadsExitStatus(self._fifo, self._threads)
        #print "Ui::destroy: Closing Fifo"
        os.close(self._fifo)
        #print "Ui::destroy: Closed, destroy called"
        gtk.mainquit()
        #print "Done"

    def _showVersion(self):
        self._stat = self.get_widget("MainStatBar")
        self._stat.set_default("AirTraffic %s" % (setup.version()))


class MainWin(abswindow.AbsWindow):
    WindowNickName = "main"
    def _build(self):
        print "Building main window"
        self._ui = abswindow.WindowList().getUserInterface()
        # external signals
        self._window.connect('destroy', self._ui.destroy)

        # connectors. 
        w = self._ui.get_widget("menuStart")
        w.connect('activate', self.showSelectWin)
        w = self._ui.get_widget("menuExit")
        w.connect('activate', self._ui.destroy)

        w = self._ui.get_widget("menuPreferences") 
        w.connect('activate', self.showPrefWin)
        w = self._ui.get_widget("menuControls")
        w.connect('activate', self.showControlWin)
        w = self._ui.get_widget("menuFlights") 
        w.connect('activate', self.showFlightWin)
        w = self._ui.get_widget("menuDetails") 
        w.connect('activate', self.showDetailWin)

        w = self._ui.get_widget("menuAbout")
        w.connect('activate', self.showAboutWin)

    def showSelectWin(self, widget):
        abswindow.WindowList().showWindow('select')
    def showControlWin(self, widget):
        abswindow.WindowList().showWindow('control')
    def showFlightWin(self, widget):
        abswindow.WindowList().showWindow('flight')
    def showDetailWin(self, widget):
        abswindow.WindowList().showWindow('detail')
    def showPrefWin(self, widget):
        abswindow.WindowList().showWindow('pref')
    def showAboutWin(self, widget):
        abswindow.WindowList().showWindow('about')
    def _windowCloseButton(self, widget, event):
        self._ui.destroy(widget)
        
class PrefWin(abswindow.AbsWindow):
    WindowNickName = "pref"
    def _build(self):
        self._ui = abswindow.WindowList().getUserInterface()
                
        # connectors
        w = self._ui.get_widget("PrefWinOk")
        w.connect('clicked', self.butOkay)
        w = self._ui.get_widget("PrefWinCancel")
        w.connect('clicked', self.butCancel)

        # signals
        self._window.connect('destroy', self.butCancel)

        # Set values.
        w = self._ui.get_widget("PrefWinMetaURL")
        w.set_text(setup.url_meta_server)

    def butOkay(self, widget):
        self._window.hide()
    def butCancel(self, widget):
        self._window.hide()
        return 1

class FlightWin(abswindow.AbsWindow):
    WindowNickName = "flight"

class DetailWin(abswindow.AbsWindow):
    WindowNickName = "detail"

class AboutWin(abswindow.AbsWindow):
    WindowNickName = "about"
        
# $Log: ui.py,v $
# Revision 1.14  2002/02/17 22:00:19  marijn
# Added dialog and about windows
#
# Revision 1.13  2002/02/07 12:51:36  marijn
# Redid window referencing
#
# Revision 1.12  2002/02/02 17:23:57  marijn
# Updated copyright information
#
# Revision 1.11  2002/01/24 14:10:18  marijn
# Silenced overtly verbose message
#
# Revision 1.10  2002/01/16 18:09:54  marijn
# Changed from named-pipes to normal ones.
# Added AbsClient.executeWithDelay() that executes the event later.
# Fixed some inter-threading bugs. the Exception de-syncing is still here.
#
# Revision 1.9  2002/01/14 15:50:26  marijn
# Added network to Ui support via Fifo. General Cleaning up
#
# Revision 1.8  2001/12/26 02:33:59  marijn
# Added SimClient to threaded model
#
# Revision 1.7  2001/12/25 02:07:11  marijn
# sim.getState() no longer needed.
#
# Revision 1.6  2001/12/10 22:15:14  marijn
# Changed threading implementation
#
# Revision 1.5  2001/12/10 00:08:54  marijn
# moved SelectWin() to selectwin.py
#
# Revision 1.4  2001/12/06 06:49:43  marijn
# MetaClient now has a seperate thread.
#
# Revision 1.3  2001/11/27 21:03:35  marijn
# Spawns thread for server comunications.
#
# Revision 1.2  2001/11/19 00:25:39  marijn
# py merged back into HEAD
#
# Revision 1.1.2.5  2001/11/16 04:00:12  marijn
# Client connects to simserver
#
# Revision 1.1.2.4  2001/11/11 08:03:56  marijn
# Shows registered simServers in selector window.
#
# Revision 1.1.2.3  2001/11/07 12:02:16  marijn
# More Ui code, moved meta server invocation to functional location
#
# Revision 1.1.2.2  2001/11/05 05:34:58  marijn
# Client now talks to meta server.
#
# Revision 1.1.2.1  2001/10/30 19:08:54  marijn
# Initial python branch commit
#
